@@ -27,7 +27,7 @@ use crate::resource::{
 use crate::scene::{PlanarSceneNode, SceneNode};
 use crate::text::{Font, TextRenderer};
 use crate::window::canvas::CanvasSetup;
-use crate::window::{Canvas, State};
+use crate::window::{Canvas, RenderLoopClosure, State};
 use image::imageops;
 use image::{GenericImage, Pixel};
 use image::{ImageBuffer, Rgb};
@@ -36,6 +36,8 @@ use ncollide3d::procedural::TriMesh;
 #[cfg(feature = "conrod")]
 use std::collections::HashMap;
 
+use super::window_cache::WindowCache;
+
 static DEFAULT_WIDTH: u32 = 800u32;
 static DEFAULT_HEIGHT: u32 = 600u32;
 
@@ -57,13 +59,33 @@ impl ConrodContext {
     }
 }
 
+// Note: this struct, and the RenderLoopClosure trait it implements, were created solely to control
+// the drop order of its members.
+// Since the Window contains the OpenGL context, it must be dropped _after_ the state, and there's
+// no way to control that in a plain old closure.
+struct RenderLoopClosureImpl<S: State, F: Fn(f64, &mut Window, &mut S) -> bool> {
+    pub state: S,
+    pub window: Window,
+    pub closure: F,
+}
+
+impl<S, F> RenderLoopClosure for RenderLoopClosureImpl<S, F>
+where
+    S: State,
+    F: Fn(f64, &mut Window, &mut S) -> bool + 'static,
+{
+    fn call(&mut self, x: f64) -> bool {
+        (self.closure)(x, &mut self.window, &mut self.state)
+    }
+}
+
 /// Structure representing a window and a 3D scene.
 ///
 /// This is the main interface with the 3d engine.
 pub struct Window {
     events: Rc<Receiver<WindowEvent>>,
     unhandled_events: Rc<RefCell<Vec<WindowEvent>>>,
-    max_dur_per_frame: Option<Duration>,
+    min_dur_per_frame: Option<Duration>,
     scene: SceneNode,
     scene2: PlanarSceneNode,
     light_mode: Light, // FIXME: move that to the scene graph
@@ -84,6 +106,12 @@ pub struct Window {
     canvas: Canvas,
 }
 
+impl Drop for Window {
+    fn drop(&mut self) {
+        WindowCache::clear();
+    }
+}
+
 impl Window {
     /// Indicates whether this window should be closed.
     #[inline]
@@ -113,7 +141,7 @@ impl Window {
     /// Sets the maximum number of frames per second. Cannot be 0. `None` means there is no limit.
     #[inline]
     pub fn set_framerate_limit(&mut self, fps: Option<u64>) {
-        self.max_dur_per_frame = fps.map(|f| {
+        self.min_dur_per_frame = fps.map(|f| {
             assert!(f != 0);
             Duration::from_millis(1000 / f)
         })
@@ -188,6 +216,8 @@ impl Window {
     }
 
     /// Set the size of all subsequent points to be drawn until the next time this function is envoked.
+    ///
+    /// Unfortunately, not all point sizes are supported by all graphics drivers.
     #[inline]
     pub fn set_point_size(&mut self, pt_size: f32) {
         self.point_renderer.set_point_size(pt_size);
@@ -527,10 +557,11 @@ impl Window {
         let canvas = Canvas::open(title, hide, width, height, setup, event_send);
 
         init_gl();
+        WindowCache::populate();
 
         let mut usr_window = Window {
             should_close: false,
-            max_dur_per_frame: None,
+            min_dur_per_frame: None,
             canvas,
             events: Rc::new(event_receive),
             unhandled_events: Rc::new(RefCell::new(Vec::new())),
@@ -910,8 +941,12 @@ impl Window {
     }
 
     /// Runs the render and event loop until the window is closed.
-    pub fn render_loop<S: State>(mut self, mut state: S) {
-        Canvas::render_loop(move |_| self.do_render_with_state(&mut state))
+    pub fn render_loop<S: State>(self, state: S) {
+        Canvas::render_loop(RenderLoopClosureImpl {
+            window: self,
+            state,
+            closure: |_, window, state| window.do_render_with_state(state),
+        });
     }
 
     /// Render one frame using the specified state.
@@ -1119,7 +1154,7 @@ impl Window {
         #[cfg(not(target_arch = "wasm32"))]
         {
             // Limit the fps if needed.
-            if let Some(dur) = self.max_dur_per_frame {
+            if let Some(dur) = self.min_dur_per_frame {
                 let elapsed = self.curr_time.elapsed();
                 if elapsed < dur {
                     std::thread::sleep(dur - elapsed);
